<?php
	/*--Database Handling Class--*/	

	class wph_db {

		//Abbreviation of new object
		protected $abvr = "idesign";

		//Zip obj instance
		public $our_zip = null;

		//Get count of total short codes
		function get_total () {
			$t = get_option(($this->abvr)."_total_sc", false);
			if ($t===false) {
				update_option(($this->abvr)."_total_sc", 0);
				$t = 0;
			}
			return $t;
		}
	
		function set_total ($i) {
			if ($i ==="del") delete_option(($this->abvr)."_total_sc");
			else update_option(($this->abvr)."_total_sc", $i);
		}

		//Set new short code
		function set_sc ($grid, $i) { //$grid = grid array, $i = index 
			if (is_numeric($i)) {
				$t = ($this->get_total())+1;
				if (!isset($i) || !is_numeric($i) || $i>($t-1)) $i = $t;//add as latest by def
				$mid = "_sc_";
				if ($i === $t) $this->set_total($t);//update total
			} else {//if the name is non-numeric the it is something like "settings"
				$mid = "_";//eg for settings
			}
			update_option(($this->abvr).$mid.$i, json_encode($grid));//update settings
			return $i;
		}
		
		//Get a short code		
		function get_sc ($n) {
			$mid = (is_numeric($n)) ? "_sc_": "_" ;
			$op = get_option(($this->abvr).$mid.$n, false);
			if (is_string($op)) $op = json_decode($op, true);//legacy json check
			return $op;
		}

		//Iterate over scs across a range with callback fn
		function iterate ($f, $r=false) { //$r = array(10,15), $f is callback
			if (!$r) $r = array(1 , $this->get_total()); //cover over all if range is not set
			for ($c=$r[0];$c<=$r[1];$c++) {
				$s = $this->get_sc($c);
				if (isset($s) && $s && isset($f))
				call_user_func_array($f, array($s,$c)); //the set and index are passed to callback
			}
		}
		
		//Modify sets
		function modify (&$design, $param) {
			foreach ($param as $val) {
				if ($val["type"] === "modify_addresses") {
					$this->modify_addresses($design, $val["param"]);
				}
			}
		}
		
		//Modify image and link addresses
		function modify_addresses (&$design, $param) {

			$replace_this = $param["replace_this"];			
			$replace_with = $param["replace_with"];

			//iterate over each of the properties, if they are certain type of property then modify
			if (!empty($design["bg-image"])) $design["bg-image"] = str_replace($replace_this, $replace_with, $design["bg-image"]);
			
			if (empty($design["sets"])) return;
			foreach($design["sets"] as &$set_and_tiles) {
				if (!empty($set_and_tiles['set']) && !empty($set_and_tiles['set']["background-image"])) $set_and_tiles['set']["background-image"] = str_replace($replace_this, $replace_with, $set_and_tiles['set']["background-image"]);
				if (empty($set_and_tiles["tiles"])) continue;
				foreach($set_and_tiles["tiles"] as $tile_index=>&$tile) {
					if (!is_numeric($tile_index)) continue;
					//style
					if (!empty($tile["style"]) && !empty($tile["style"]["background-image"]))  
					$tile["style"]["background-image"] = str_replace($replace_this, $replace_with, $tile["style"]["background-image"]);
					//image
					if (!empty($tile["image"]) && !empty($tile["image"]["content"]))  
					$tile["image"]["content"] = str_replace($replace_this, $replace_with, $tile["image"]["content"]);
					//image hover
					if (!empty($tile["image-hover"]) && !empty($tile["image-hover"]["content"]))  
					$tile["image-hover"]["content"] = str_replace($replace_this, $replace_with, $tile["image-hover"]["content"]);
					//main link
					if (!empty($tile["link"]) ) {
						if (!empty($tile["link"]["content"])) $tile["link"]["content"] = str_replace($replace_this, $replace_with, $tile["link"]["content"]);
						foreach($tile["link"] as $key=>&$val) {
							if (!is_numeric($key)) continue;
							if (empty($val["style"]) || empty($val["style"]["url"]) || ($val['style']["url"]=== "wmt_main_link" || $val['style']["url"]=== "wmt_lightbox" || $val['style']["url"]=== "woo_page" || $val['style']["url"]=== "woo_cart") ) continue;
							$val['style']["url"] = str_replace($replace_this, $replace_with, $val['style']["url"]);
						}
					}
					//video
					if (!empty($tile["video"]) && !empty($tile["video"]["content"])) $tile["video"]["content"] = str_replace($replace_this, $replace_with, $tile["video"]["content"]);
					//lightbox
					if (!empty($tile["lightbox"]) && !empty($tile["link"]["lightbox"])) $tile["link"]["lightbox"] = str_replace($replace_this, $replace_with, $tile["link"]["lightbox"]);
				}
			}

		}

		//Del target sc
		function del_sc ($s) {
			delete_option(($this->abvr)."_sc_".$s);
		}

		//Del all sc
		function del_all_scs () {
			$t = $this->get_total();
			for ($c=0;$c<=$t;$c++) {//do not use 0 
				$this->del_sc($c);
			}
			$this->set_total('del');
			$s = get_option(($this->abvr)."_settings");
			if (is_array($s)) {
				foreach($s as $key=>$val) {
					delete_option(($this->abvr)."_".$key);
				};
			}
			delete_option(($this->abvr)."_settings");
		}

		//Create backup
		function new_backup ($f) { //$f = filename
			if(file_exists($f)) unlink($f);//delete the prev db
			$this->our_zip = new ZipArchive();
			if ($this->our_zip->open($f, ZipArchive::CREATE)!==TRUE) {
				exit("cannot open <$f>\n");
			};
			$this->iterate(array($this, 'zip_it'));
			$s = get_option($this->abvr.'_settings', false);
			if ($s) $this->zip_it($s);
			$this->our_zip->close();
		}

		//Populate zip backup
		function zip_it ($s, $c="settings") { //called as callback to iterate
			if ($s) $s = json_encode($s);
			$this->our_zip->addFromString($c.".txt", $s);
		}

		//Read backup
		function read_backup ($f, $mod_params = false) {
			$t = $this->get_total();
			$zip = zip_open($f);
			if ($zip) {
				//delete previous entries
				$this->del_all_scs();
				//write each new entry
				while ($zip_entry = zip_read($zip)) {
					//check name number
					$name  = zip_entry_name($zip_entry);
					$name = substr($name, 0 , -4);
					//open zip and read
					if (zip_entry_open($zip, $zip_entry, "r")) {
						//clean slate
						$buf = zip_entry_read($zip_entry, zip_entry_filesize($zip_entry));
						$buf = json_decode($buf, true);
						if ($mod_params) $this->modify($buf, $mod_params);
						//write to options
						$this->set_sc ($buf, $name);//creates a new option by that name
						if ($name==="settings" && is_array($buf)) {//each key under "settings" gets its own separate option
							foreach($buf as $key=>$val) {
								update_option($this->abvr.'_'.$key, $val);
							}
						}
						zip_entry_close($zip_entry);
					}
				}
				zip_close($zip);
			}
		}
		
	}
?>